@@ -25,8 +25,10 @@ from . import utils
 import sys
 import email
 import email.errors
+import email.utils
 import glob
 import os
+import re
 import urllib.parse
 import textwrap
 # SOAP interface to Debian BTS
@@ -36,6 +38,16 @@ from contextlib import suppress
 
 from . import checkversions
 
+# This try..except is only useful for developers running reportbug from
+# the source tree; for normal users it will never fail.
+try:
+    from .data import arch_descriptions
+    from .data import arch_status
+    from .data import arch_usertags
+    from .data import order_arches
+except ModuleNotFoundError:
+    raise ModuleNotFoundError('reportbug data cache not found, please run: make convert-data')
+
 
 class Error(Exception):
     pass
@@ -206,6 +218,57 @@ debother = {
 }
 
 
+default_arches = set()
+
+
+def _select_multiple_architectures(ui, arches):
+    """Interactively select from a menu of architectures & descriptions
+
+    Parameters
+    ----------
+    ui : reportbug UI object
+        reportbug UI
+    arches : list of str
+        the architectures to display in the menu
+
+    Returns
+    -------
+    None or list of str
+        None when cancelled, or the selected architectures otherwise
+    """
+    return ui.select_multiple(
+        'Which architectures would you like to select?',
+        {arch: arch_descriptions.get(arch, '') for arch in arches},
+        'Please select arches: ',
+        default=default_arches,
+        order=order_arches,
+    )
+
+
+def _apply_architectures(ui, pseudos, arches):
+    users = {}
+    xccs = set()
+    for arch in arches:
+        if arch not in arch_usertags:
+            continue
+        _url, user, usertags, xcc = arch_usertags[arch]
+        if user not in users:
+            users[user] = set()
+        if user and usertags:
+            users[user].update(usertags)
+        if xcc:
+            xccs.add(xcc)
+    for user, usertags in users.items():
+        pseudos.append('User: %s' % user)
+        pseudos.append('Usertags: %s' % ' '.join(sorted(usertags)))
+    if xccs:
+        pseudos.append('X-Debbugs-Cc: %s' % ', '.join(sorted(xccs)))
+        suffix = '@lists.debian.org'
+        emails = [xcc.removesuffix(suffix) for xcc in sorted(xccs)]
+        emails = ', '.join(emails)
+        ui.log_message('Your report will be carbon-copied to %s.\n' % emails)
+
+
 def check_package_info(package):
     """
     Get package information
@@ -456,11 +519,13 @@ def handle_debian_ftp(package, bts, ui, fromaddr, timeout, online=True, http_pro
             'yN', {'y': 'This is a partial (specific architectures) removal.',
                    'n': 'This removal is for all architectures.'})
         if partial == 'y':
-            prompt = 'Please enter the arch list separated by a space: '
-            archs = ui.get_string(prompt)
+            # ftp-master only handles removals for official architectures
+            archs = {arch for arch, official in arch_status.items() if official}
+            archs = _select_multiple_architectures(ui, archs)
             if not archs:
                 ui.long_message('Partial removal requests must have a list of architectures.\n')
                 raise SystemExit
+            _apply_architectures(ui, pseudos, archs)
 
         if suite == 'testing' and archs:
             ui.long_message('Partial removal for testing; forcing suite to '
@@ -471,6 +536,7 @@ def handle_debian_ftp(package, bts, ui, fromaddr, timeout, online=True, http_pro
                    'converted in one for unstable'
 
         if archs:
+            archs = ' '.join(archs)
             if suite != 'unstable':
                 subject = 'RM: %s/%s [%s] -- %s; %s' % (package, suite, archs, tag, reason)
             else:
@@ -636,10 +702,19 @@ def handle_debian_release(package, bts, ui, fromaddr, timeout, online=True, http
                                 'be automatically removed from testing too. Please re-run '
                                 'reportbug against ftp.debian.org package.')
                 raise SystemExit
-            prompt = 'Please enter the arch list separated by a space: '
-            archs = ui.get_string(prompt)
+            # The release team only handles binNMUs for official architectures
+            # changelog-only source rebuilds are currently not supported by Debian
+            archs = {arch for arch, official in arch_status.items() if official and arch != 'all'}
+            archs = _select_multiple_architectures(ui, archs)
             if not archs:
                 ui.long_message('No architecture specified, skipping...')
+            else:
+                _apply_architectures(ui, pseudos, archs)
+                if utils.has_multiarch_same(package):
+                    ui.long_message(f'Source package {package} has some Multi-Arch: same binary packages. '
+                                    'Requesting binNMUs for only some arches causes version skew. '
+                                    'Converting to a request to binNMU all architectures instead.')
+                    archs = []
 
     if tag == 'binnmu':
         suite = ui.menu("For which suite are you requesting this binNMU?",
@@ -668,6 +743,8 @@ def handle_debian_release(package, bts, ui, fromaddr, timeout, online=True, http
     if tag == 'binnmu':
         reason = ui.get_string("binNMU changelog entry: ")
         subject = "nmu: %s_%s" % (package, version)
+        archs = archs if archs else []
+        archs = ' '.join(archs)
         body = "nmu %s_%s . %s . %s . -m \"%s\"\n" % (package, version, archs or "ANY", suite, reason)
     elif tag == 'transition':
         subject = 'transition: %s' % (package)
@@ -1308,14 +1385,6 @@ CLASSES = {
 
 CLASSLIST = ['sw-bug', 'doc-bug', 'change-request']
 
-CRITICAL_TAGS = {
-    'security': 'This problem is a security vulnerability in Debian.',
-}
-
-EXPERT_TAGS = {
-    'security': 'This problem is a security vulnerability in Debian.',
-}
-
 TAGS = {
     'patch': 'You are including a patch to fix this problem.',
     'upstream': 'This bug applies to the upstream part of the package.',
@@ -1326,10 +1395,155 @@ TAGS = {
     'a11y': 'This bug affects accessibility for users with disabilities.',
     'newcomer': 'This bug has a known solution but the maintainer requests someone else implement it.',
     'ftbfs': 'The package fails to build from source.',
+    'security': 'This problem is a security vulnerability in Debian.',
+    'arch': 'This bug only applies to specific architectures.',
 }
 
 
-def get_tags(severity='', mode=utils.MODE.NOVICE):
+def _selected_arch_tag(ui, arches):
+    global default_arches
+    default_arches |= arches
+    if arches:
+        arches = ' '.join(sorted(arches))
+        msg = 'Adding arch usertags to this report for: %s\n' % arches
+        ui.log_message(msg)
+    return bool(default_arches)
+
+
+def _select_arch_tag_based_on_package(ui, package):
+    return package in 'cdimage.debian.org installation-reports'.split()
+
+
+def _detect_subject_architectures(subject):
+    nonword = r'''-!"#$%&'()*+,./:;<=>?@[\]^_`{|}~\s'''
+    detected_arches = set()
+    arches = {arch for arch in arch_usertags}
+    for arch in arches:
+        regex = r'(^|[%s])%s($|[%s])' % (nonword, re.escape(arch), nonword)
+        if re.search(regex, subject) is not None:
+            detected_arches.add(arch)
+    return detected_arches
+
+
+def _select_arch_tag_based_on_subject(ui, subject):
+    ftbfs = subject and 'FTBFS' in subject.upper() or 'fails to build' in subject
+    arches = _selected_arch_tag(ui, _detect_subject_architectures(subject))
+    return ftbfs or arches
+
+
+def _detect_xcc_architectures(xccs):
+    xccs = email.utils.getaddresses(xccs)
+    xccs = [email for name, email in xccs]
+    detected_arches = set()
+    for arch, (_url, _user, _usertags, xcc) in arch_usertags.items():
+        if xcc in xccs:
+            detected_arches.add(arch)
+    return detected_arches
+
+
+def _select_arch_tag_based_on_xcc(ui, xcc):
+    return _selected_arch_tag(ui, _detect_xcc_architectures(xcc))
+
+
+def _select_arch_tag_based_on_justification(ui, justification):
+    return justification and 'fails to build from source' in justification
+
+
+def _select_arch_tag_based_on_tags(ui, tags):
+    return bool({'ftbfs', 'd-i'} & set(tags))
+
+
+SELECT_TAGS = {
+    'package': {
+        'arch': _select_arch_tag_based_on_package,
+    },
+    'subject': {
+        'arch': _select_arch_tag_based_on_subject,
+    },
+    'xcc': {
+        'arch': _select_arch_tag_based_on_xcc,
+    },
+    'justification': {
+        'arch': _select_arch_tag_based_on_justification,
+    },
+    'tags': {
+        'arch': _select_arch_tag_based_on_tags,
+    },
+}
+
+
+def select_tags(ui, taglist, check, value):
+    """Select tags based on checks
+
+    Parameters
+    ----------
+    ui : reportbug UI object
+        reportbug UI
+    taglist : list of str
+        list of tags to modify
+    check : str
+        type of check
+    value : str
+        value to check
+
+    Returns
+    -------
+    list of str
+        new taglist
+    """
+    for tag, select_tag in SELECT_TAGS[check].items():
+        if select_tag(ui, value):
+            taglist += [tag]
+    return taglist
+
+
+SELECT_OTHER_TAGS = {
+    'ftbfs': 'arch',
+    'd-i': 'arch',
+}
+
+
+def select_other_tags():
+    """Suggest tags to select when other tags are selected
+
+    Parameters
+    ----------
+
+    Returns
+    -------
+    {str : str, ...}
+        dictionary of tags to find and select
+    """
+    return SELECT_OTHER_TAGS
+
+
+def _show_security_tag(severity, mode):
+    return severity in ('grave', 'critical', 'serious') \
+        or mode > utils.MODE.STANDARD
+
+
+def _show_newcomer_tag(severity, mode):
+    # do not show the newcomer tag in novice mode
+    return mode >= utils.MODE.STANDARD
+
+
+def _show_arch_tag(severity, mode):
+    return mode >= utils.MODE.STANDARD
+
+
+SHOW_TAGS = {
+    'security': _show_security_tag,
+    'newcomer': _show_newcomer_tag,
+    'arch': _show_arch_tag,
+}
+
+
+VIRTUAL_TAGS = {
+    'arch',
+}
+
+
+def get_tags(severity='', mode=utils.MODE.NOVICE, virtual=True):
     """
     Return list of tags supported by the bug tracking system
 
@@ -1351,18 +1565,119 @@ def get_tags(severity='', mode=utils.MODE.NOVICE):
     """
     tags = TAGS.copy()
 
-    if severity in ('grave', 'critical', 'serious'):
-        tags.update(CRITICAL_TAGS)
-
-    if mode > utils.MODE.STANDARD:
-        tags.update(EXPERT_TAGS)
-    elif mode < utils.MODE.STANDARD and 'newcomer' in tags:
-        # do not show the newcomer tag in novice mode
-        del tags['newcomer']
+    for tag, show_tag in SHOW_TAGS.items():
+        if (not virtual and tag in VIRTUAL_TAGS) or \
+           not show_tag(severity, mode):
+            tags.pop(tag, None)
 
     return tags
 
 
+SORT_TAGS = {
+    'arch': 100,
+}
+
+
+def sort_tags(tags):
+    """Sort tags based on checks
+
+    Parameters
+    ----------
+    tags : list of str
+        list of tags
+
+    Returns
+    -------
+    list of str
+        sorted list of tags
+    """
+    return sorted(tags, key=lambda item: (SORT_TAGS.get(item, 0), item))
+
+
+def _handle_di_tag(mode, ui, pseudos, taglist, locals):
+    listcc = locals['listcc']
+    maintainer = locals['maintainer']
+    debian_boot = 'debian-boot@lists.debian.org'
+    emails = email.utils.getaddresses(maintainer)
+    emails = [email for name, email in emails]
+    if debian_boot not in emails and debian_boot not in listcc:
+        listcc += [debian_boot]
+        ui.log_message('Your report will be carbon-copied to debian-boot.\n')
+
+
+def _apply_di_architectures(ui, pseudos, arches):
+    pseudos.append('User: debian-boot@lists.debian.org')
+    pseudos.append('Usertags: %s' % ' '.join(sorted(arches)))
+
+
+SHOW_ARCHES = {
+    # Show no architectures for novice mode
+    utils.MODE.NOVICE:
+        set(),
+
+    # Show only official architectures for standard mode
+    utils.MODE.STANDARD:
+        {arch for arch in arch_usertags if arch_status.get(arch)},
+
+    # Show only current architectures for advanced mode
+    utils.MODE.ADVANCED:
+        {arch for arch in arch_usertags if arch in arch_status},
+
+    # Show all usertags architectures for expert mode
+    utils.MODE.EXPERT:
+        {arch for arch in arch_usertags},
+}
+
+
+def _handle_arch_tag(mode, ui, pseudos, taglist, _locals):
+    taglist.remove('arch')
+    # Show system architecture(s)
+    if mode >= utils.MODE.STANDARD:
+        arches = utils.get_arch()
+        arches += ', ' + utils.get_multiarch()
+        arches = arches.split(',')
+        arches = {arch.strip() for arch in arches}
+        arches = {arch for arch in arches if arch in arch_usertags}
+    arches |= SHOW_ARCHES.get(mode, set())
+    arches = _select_multiple_architectures(ui, arches)
+    if arches:
+        _apply_architectures(ui, pseudos, arches)
+        if 'd-i' in taglist:
+            _apply_di_architectures(ui, pseudos, arches)
+    return arches
+
+
+HANDLE_TAGS = {
+    'd-i': _handle_di_tag,
+    'arch': _handle_arch_tag,
+}
+
+
+def handle_tags(mode, ui, pseudos, taglist, locals):
+    """Handle tags with extra requirements
+
+    Parameters
+    ----------
+    mode : int
+        reportbug user mode level
+    ui : reportbug UI object
+        reportbug UI
+    pseudos : list of str
+        list of pseudo-headers
+    taglist : list of str
+        list of tags
+
+    Returns
+    -------
+    [str, ...]
+        list of tag names
+    """
+    for tag, handle_tag in HANDLE_TAGS.items():
+        if tag in taglist:
+            handle_tag(mode, ui, pseudos, taglist, locals)
+    return taglist
+
+
 def yn_bool(setting):
     """
     Turn `setting` into equivalent string
